@@ -706,11 +706,11 @@ pub(crate) fn fadvise(
     let err = unsafe { libc_posix_fadvise(borrowed_fd(fd), offset, len, advice as libc::c_int) };
 
     // `posix_fadvise` returns its error status rather than using `errno`.
-    return if err == 0 {
+    if err == 0 {
         Ok(())
     } else {
         Err(io::Error(err))
-    };
+    }
 }
 
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
@@ -1728,27 +1728,6 @@ pub(crate) fn shutdown(sockfd: BorrowedFd<'_>, how: Shutdown) -> io::Result<()>
     unsafe { ret(libc::shutdown(borrowed_fd(sockfd), how as c_int)) }
 }
 
-#[cfg(not(any(target_os = "redox", target_os = "wasi")))]
-pub(crate) fn getsockopt_socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
-    let mut buffer = MaybeUninit::<SocketType>::uninit();
-    let mut out_len = size_of::<SocketType>() as libc::socklen_t;
-    unsafe {
-        ret(libc::getsockopt(
-            borrowed_fd(fd),
-            libc::SOL_SOCKET,
-            libc::SO_TYPE,
-            buffer.as_mut_ptr().cast::<libc::c_void>(),
-            &mut out_len,
-        ))?;
-        assert_eq!(
-            out_len as usize,
-            size_of::<SocketType>(),
-            "unexpected SocketType size"
-        );
-        Ok(buffer.assume_init())
-    }
-}
-
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 pub(crate) fn getsockname(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
     unsafe {
@@ -1842,7 +1821,6 @@ pub(crate) fn clock_gettime(id: ClockId) -> Timespec {
 
 #[cfg(not(target_os = "wasi"))]
 #[inline]
-#[must_use]
 pub(crate) fn clock_gettime_dynamic(id: DynamicClockId) -> io::Result<Timespec> {
     let mut timespec = MaybeUninit::<Timespec>::uninit();
     unsafe {
@@ -1894,7 +1872,6 @@ pub(crate) fn clock_gettime_dynamic(id: DynamicClockId) -> io::Result<Timespec>
     target_os = "wasi",
 )))]
 #[inline]
-#[must_use]
 pub(crate) fn clock_nanosleep_relative(id: ClockId, request: &Timespec) -> NanosleepRelativeResult {
     let mut remain = MaybeUninit::<Timespec>::uninit();
     let flags = 0;
@@ -1929,7 +1906,6 @@ pub(crate) fn clock_nanosleep_absolute(id: ClockId, request: &Timespec) -> io::R
 
 #[cfg(not(target_os = "redox"))]
 #[inline]
-#[must_use]
 pub(crate) fn nanosleep(request: &Timespec) -> NanosleepRelativeResult {
     let mut remain = MaybeUninit::<Timespec>::uninit();
     unsafe {
@@ -2250,3 +2226,403 @@ pub(crate) fn setpriority_process(pid: Pid, priority: i32) -> io::Result<()> {
         ))
     }
 }
+
+#[cfg(not(any(target_os = "redox", target_os = "wasi")))]
+pub(crate) mod sockopt {
+    use crate::net::sockopt::Timeout;
+    #[cfg(not(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "netbsd"
+    )))]
+    use crate::net::Ipv6Addr;
+    use crate::net::{Ipv4Addr, SocketType};
+    use crate::{as_mut_ptr, io};
+    use io_lifetimes::BorrowedFd;
+    use std::convert::TryInto;
+    use std::os::raw::{c_int, c_uint};
+    use std::time::Duration;
+
+    #[inline]
+    fn getsockopt<T>(fd: BorrowedFd<'_>, level: i32, optname: i32) -> io::Result<T> {
+        use super::*;
+        unsafe {
+            let mut value = MaybeUninit::<T>::uninit();
+            let mut optlen = std::mem::size_of::<T>().try_into().unwrap();
+            ret(libc::getsockopt(
+                borrowed_fd(fd),
+                level,
+                optname,
+                as_mut_ptr(&mut value).cast(),
+                &mut optlen,
+            ))?;
+            assert_eq!(
+                optlen as usize,
+                size_of::<T>(),
+                "unexpected getsockopt size"
+            );
+            Ok(value.assume_init())
+        }
+    }
+
+    #[inline]
+    fn setsockopt<T>(fd: BorrowedFd<'_>, level: i32, optname: i32, value: T) -> io::Result<()> {
+        use super::*;
+        unsafe {
+            let optlen = std::mem::size_of::<T>().try_into().unwrap();
+            ret(libc::setsockopt(
+                borrowed_fd(fd),
+                level,
+                optname,
+                as_ptr(&value).cast(),
+                optlen,
+            ))
+        }
+    }
+
+    #[inline]
+    pub(crate) fn get_socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
+        getsockopt(fd, libc::SOL_SOCKET as _, libc::SO_TYPE)
+    }
+
+    #[inline]
+    pub(crate) fn set_socket_reuseaddr(fd: BorrowedFd<'_>, reuseaddr: bool) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::SOL_SOCKET as _,
+            libc::SO_REUSEADDR,
+            from_bool(reuseaddr),
+        )
+    }
+
+    #[inline]
+    pub(crate) fn set_socket_broadcast(fd: BorrowedFd<'_>, broadcast: bool) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::SOL_SOCKET as _,
+            libc::SO_BROADCAST,
+            from_bool(broadcast),
+        )
+    }
+
+    #[inline]
+    pub(crate) fn get_socket_broadcast(fd: BorrowedFd<'_>) -> io::Result<bool> {
+        getsockopt(fd, libc::SOL_SOCKET as _, libc::SO_BROADCAST)
+    }
+
+    #[inline]
+    pub(crate) fn set_socket_linger(
+        fd: BorrowedFd<'_>,
+        linger: Option<Duration>,
+    ) -> io::Result<()> {
+        let linger = libc::linger {
+            l_onoff: linger.is_some() as c_int,
+            l_linger: linger.unwrap_or_default().as_secs() as c_int,
+        };
+        setsockopt(fd, libc::SOL_SOCKET as _, libc::SO_LINGER, linger)
+    }
+
+    #[inline]
+    pub(crate) fn get_socket_linger(fd: BorrowedFd<'_>) -> io::Result<Option<Duration>> {
+        let linger: libc::linger = getsockopt(fd, libc::SOL_SOCKET as _, libc::SO_LINGER)?;
+        Ok((linger.l_onoff != 0).then(|| Duration::from_secs(linger.l_linger as u64)))
+    }
+
+    #[cfg(any(target_os = "android", target_os = "linux"))]
+    #[inline]
+    pub(crate) fn set_socket_passcred(fd: BorrowedFd<'_>, passcred: bool) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::SOL_SOCKET as _,
+            libc::SO_PASSCRED,
+            from_bool(passcred),
+        )
+    }
+
+    #[cfg(any(target_os = "android", target_os = "linux"))]
+    #[inline]
+    pub(crate) fn get_socket_passcred(fd: BorrowedFd<'_>) -> io::Result<bool> {
+        getsockopt(fd, libc::SOL_SOCKET as _, libc::SO_PASSCRED)
+    }
+
+    #[inline]
+    pub(crate) fn set_socket_timeout(
+        fd: BorrowedFd<'_>,
+        id: Timeout,
+        timeout: Option<Duration>,
+    ) -> io::Result<()> {
+        let timeout = match timeout {
+            Some(timeout) => {
+                if timeout == Duration::ZERO {
+                    return Err(io::Error::INVAL);
+                }
+
+                let tv_sec = timeout.as_secs().try_into();
+                #[cfg(not(all(target_arch = "x86_64", target_pointer_width = "32")))]
+                let tv_sec = tv_sec.unwrap_or(std::os::raw::c_long::MAX);
+                #[cfg(all(target_arch = "x86_64", target_pointer_width = "32"))]
+                let tv_sec = tv_sec.unwrap_or(i64::MAX);
+
+                let mut timeout = libc::timeval {
+                    tv_sec,
+                    tv_usec: timeout.subsec_micros() as _,
+                };
+                if timeout.tv_sec == 0 && timeout.tv_usec == 0 {
+                    timeout.tv_usec = 1;
+                }
+                timeout
+            }
+            None => libc::timeval {
+                tv_sec: 0,
+                tv_usec: 0,
+            },
+        };
+        let optname = match id {
+            Timeout::Recv => libc::SO_RCVTIMEO,
+            Timeout::Send => libc::SO_SNDTIMEO,
+        };
+        setsockopt(fd, libc::SOL_SOCKET, optname, timeout)
+    }
+
+    #[inline]
+    pub(crate) fn get_socket_timeout(
+        fd: BorrowedFd<'_>,
+        id: Timeout,
+    ) -> io::Result<Option<Duration>> {
+        let optname = match id {
+            Timeout::Recv => libc::SO_RCVTIMEO,
+            Timeout::Send => libc::SO_SNDTIMEO,
+        };
+        let timeout: libc::timeval = getsockopt(fd, libc::SOL_SOCKET, optname)?;
+        if timeout.tv_sec == 0 && timeout.tv_usec == 0 {
+            Ok(None)
+        } else {
+            Ok(Some(
+                Duration::from_secs(timeout.tv_sec as u64)
+                    + Duration::from_micros(timeout.tv_usec as u64),
+            ))
+        }
+    }
+
+    #[inline]
+    pub(crate) fn set_ip_ttl(fd: BorrowedFd<'_>, ttl: u32) -> io::Result<()> {
+        setsockopt(fd, libc::IPPROTO_IP as _, libc::IP_TTL, ttl)
+    }
+
+    #[inline]
+    pub(crate) fn get_ip_ttl(fd: BorrowedFd<'_>) -> io::Result<u32> {
+        getsockopt(fd, libc::IPPROTO_IP as _, libc::IP_TTL)
+    }
+
+    #[inline]
+    pub(crate) fn set_ipv6_v6only(fd: BorrowedFd<'_>, only_v6: bool) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::IPPROTO_IPV6 as _,
+            libc::IPV6_V6ONLY,
+            from_bool(only_v6),
+        )
+    }
+
+    #[inline]
+    pub(crate) fn get_ipv6_v6only(fd: BorrowedFd<'_>) -> io::Result<bool> {
+        getsockopt(fd, libc::IPPROTO_IPV6 as _, libc::IPV6_V6ONLY)
+    }
+
+    #[inline]
+    pub(crate) fn set_ip_multicast_loop(
+        fd: BorrowedFd<'_>,
+        multicast_loop: bool,
+    ) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::IPPROTO_IP as _,
+            libc::IP_MULTICAST_LOOP,
+            from_bool(multicast_loop),
+        )
+    }
+
+    #[inline]
+    pub(crate) fn get_ip_multicast_loop(fd: BorrowedFd<'_>) -> io::Result<bool> {
+        getsockopt(fd, libc::IPPROTO_IP as _, libc::IP_MULTICAST_LOOP)
+    }
+
+    #[inline]
+    pub(crate) fn set_ip_multicast_ttl(fd: BorrowedFd<'_>, multicast_ttl: u32) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::IPPROTO_IP as _,
+            libc::IP_MULTICAST_TTL,
+            multicast_ttl,
+        )
+    }
+
+    #[inline]
+    pub(crate) fn get_ip_multicast_ttl(fd: BorrowedFd<'_>) -> io::Result<u32> {
+        getsockopt(fd, libc::IPPROTO_IP as _, libc::IP_MULTICAST_TTL)
+    }
+
+    #[inline]
+    pub(crate) fn set_ipv6_multicast_loop(
+        fd: BorrowedFd<'_>,
+        multicast_loop: bool,
+    ) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::IPPROTO_IPV6 as _,
+            libc::IPV6_MULTICAST_LOOP,
+            from_bool(multicast_loop),
+        )
+    }
+
+    #[inline]
+    pub(crate) fn get_ipv6_multicast_loop(fd: BorrowedFd<'_>) -> io::Result<bool> {
+        getsockopt(fd, libc::IPPROTO_IPV6 as _, libc::IPV6_MULTICAST_LOOP)
+    }
+
+    #[inline]
+    pub(crate) fn set_ip_add_membership(
+        fd: BorrowedFd<'_>,
+        multiaddr: &Ipv4Addr,
+        interface: &Ipv4Addr,
+    ) -> io::Result<()> {
+        let mreq = to_imr(multiaddr, interface);
+        setsockopt(fd, libc::IPPROTO_IP as _, libc::IP_ADD_MEMBERSHIP, mreq)
+    }
+
+    #[cfg(not(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    )))]
+    #[inline]
+    pub(crate) fn set_ipv6_join_group(
+        fd: BorrowedFd<'_>,
+        multiaddr: &Ipv6Addr,
+        interface: u32,
+    ) -> io::Result<()> {
+        let mreq = to_ipv6mr(multiaddr, interface);
+        setsockopt(fd, libc::IPPROTO_IPV6 as _, libc::IPV6_ADD_MEMBERSHIP, mreq)
+    }
+
+    #[cfg(not(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "netbsd"
+    )))]
+    #[inline]
+    pub(crate) fn set_ip_drop_membership(
+        fd: BorrowedFd<'_>,
+        multiaddr: &Ipv4Addr,
+        interface: &Ipv4Addr,
+    ) -> io::Result<()> {
+        let mreq = to_imr(multiaddr, interface);
+        setsockopt(fd, libc::IPPROTO_IP as _, libc::IP_DROP_MEMBERSHIP, mreq)
+    }
+
+    #[cfg(not(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "netbsd",
+        target_os = "openbsd",
+    )))]
+    #[inline]
+    pub(crate) fn set_ipv6_leave_group(
+        fd: BorrowedFd<'_>,
+        multiaddr: &Ipv6Addr,
+        interface: u32,
+    ) -> io::Result<()> {
+        let mreq = to_ipv6mr(multiaddr, interface);
+        setsockopt(
+            fd,
+            libc::IPPROTO_IPV6 as _,
+            libc::IPV6_DROP_MEMBERSHIP,
+            mreq,
+        )
+    }
+
+    #[inline]
+    pub(crate) fn set_tcp_nodelay(fd: BorrowedFd<'_>, nodelay: bool) -> io::Result<()> {
+        setsockopt(
+            fd,
+            libc::IPPROTO_TCP as _,
+            libc::TCP_NODELAY,
+            from_bool(nodelay),
+        )
+    }
+
+    #[inline]
+    pub(crate) fn get_tcp_nodelay(fd: BorrowedFd<'_>) -> io::Result<bool> {
+        getsockopt(fd, libc::IPPROTO_TCP as _, libc::TCP_NODELAY)
+    }
+
+    #[inline]
+    fn to_imr(multiaddr: &Ipv4Addr, interface: &Ipv4Addr) -> libc::ip_mreq {
+        libc::ip_mreq {
+            imr_multiaddr: to_imr_addr(multiaddr),
+            imr_interface: to_imr_addr(interface),
+        }
+    }
+
+    #[inline]
+    fn to_imr_addr(addr: &Ipv4Addr) -> libc::in_addr {
+        libc::in_addr {
+            s_addr: u32::from_ne_bytes(addr.octets()),
+        }
+    }
+
+    #[cfg(not(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "netbsd"
+    )))]
+    #[inline]
+    fn to_ipv6mr(multiaddr: &Ipv6Addr, interface: u32) -> libc::ipv6_mreq {
+        libc::ipv6_mreq {
+            ipv6mr_multiaddr: to_ipv6mr_multiaddr(multiaddr),
+            ipv6mr_interface: to_ipv6mr_interface(interface),
+        }
+    }
+
+    #[cfg(not(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "netbsd"
+    )))]
+    #[inline]
+    fn to_ipv6mr_multiaddr(multiaddr: &Ipv6Addr) -> libc::in6_addr {
+        libc::in6_addr {
+            s6_addr: multiaddr.octets(),
+        }
+    }
+
+    #[cfg(target_os = "android")]
+    #[inline]
+    fn to_ipv6mr_interface(interface: u32) -> c_int {
+        interface as c_int
+    }
+
+    #[cfg(not(any(
+        target_os = "android",
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "netbsd"
+    )))]
+    #[inline]
+    fn to_ipv6mr_interface(interface: u32) -> c_uint {
+        interface as c_uint
+    }
+
+    #[inline]
+    fn from_bool(value: bool) -> c_uint {
+        value as c_uint
+    }
+}
