@@ -4,6 +4,8 @@
 
 #[cfg(target_arch = "aarch64")]
 mod aarch64;
+#[cfg(target_arch = "arm")]
+mod arm;
 #[cfg(target_arch = "riscv64")]
 mod riscv64;
 #[cfg(target_arch = "x86")]
@@ -13,6 +15,8 @@ mod x86_64;
 
 #[cfg(target_arch = "aarch64")]
 pub(in crate::imp::linux_raw) use self::aarch64::*;
+#[cfg(target_arch = "arm")]
+pub(in crate::imp::linux_raw) use self::arm::*;
 #[cfg(target_arch = "riscv64")]
 pub(in crate::imp::linux_raw) use self::riscv64::*;
 #[cfg(target_arch = "x86")]
