@@ -4,5 +4,6 @@ mod types;
 pub(super) use auxv::sysinfo_ehdr;
 pub(crate) use auxv::{linux_hwcap, page_size};
 pub use types::{
-    RawGid, RawPid, RawUid, RawUname, EXIT_FAILURE, EXIT_SIGNALED_SIGABRT, EXIT_SUCCESS,
+    MembarrierCommand, RawCpuid, RawGid, RawPid, RawUid, RawUname, EXIT_FAILURE,
+    EXIT_SIGNALED_SIGABRT, EXIT_SUCCESS,
 };
