@@ -438,6 +438,8 @@ const STN_UNDEF: u32 = 0;
 const VER_FLG_BASE: u16 = 0x1;
 const VER_DEF_CURRENT: u16 = 1;
 const STV_DEFAULT: u8 = 0;
+#[cfg(target_arch = "arm")]
+const EM_CURRENT: u16 = 40; // EM_ARM
 #[cfg(target_arch = "x86")]
 const EM_CURRENT: u16 = 3; // EM_386
 #[cfg(target_arch = "x86_64")]
